/*
 * Copyright 2005, Freie Universitaet Berlin. All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * Contributors: Georg Wittenburg
 */

#ifndef EEPROM_GEN_H
#define EEPROM_GEN_H 1

/**
 * BIG HUGE FREAKING WARNING!!!
 *
 * The typedefs below MUST define the types found in ScatterWeb.System.h as
 * corresponding types of your LOCAL architecture.
 */

typedef unsigned char		UINT8;
typedef unsigned short int	UINT16;
typedef unsigned int		UINT32;



/**
 * ANOTHER BIG HUGE FREAKING WARNING!!!
 *
 * The typedef below MUST match the one found in ScatterWeb.System.h.
 */

enum {
	True = 1,
	False = 0,
};
typedef char bool;



/**
 * ANOTHER (COMPLETELY DIFFERENT) BIG HUGE FREAKING WARNING!!!
 *
 * The typedef below MUST match the one found in ScatterWeb.Time.h, but has a
 * different name to avoid collisions with other definitions of time_t.
 */

typedef struct {
	UINT32 secs;
	UINT16 millis;
} scatterweb_time_t;



/**
 * minor warning.
 *
 * This define should point to the location of the Configuration.id field in
 * the ScatterWeb EEPROM image.
 */

#define SCATTERWEB_CONFIGURATION_ID	0x0F00



/**
 * PROBABLY THE BIGGEST HUGE FREAKING WARNING!!!
 *
 * The defines and typedefs below MUST match those found in RlsProc.h of the
 * ScatterWeb implementation of FACTS.
 */

#define RLS_MAGIC_NUMBER	0xFAC7	// Bytecode MUST start with this at RLS_MAGIC_ADDR to be recognized.

// Boundaries of our part of the EEPROM on a sensor node.
#define RLS_MAGIC_ADDR		0x1000	// Up to this address the EEPROM contains the firmware image.
#define RLS_HEADER_ADDR		0x1002	// First address of payload
#define RLS_MAX_ADDR		0xFFFF	// 64k of EEPROM.

#define RLS_FIELD_UNUSED	0x0000	// This value is to be set whenever a field of a struct is not used.

/**
 * gcc pads elements of a struct to 16 bit. Only if two 8 bit elements are
 * declared next to each other, they are put into one 16 bit slot.
 *
 * WARNING: The ScatterWeb compiler MUST show a matching behavior, otherwise
 * parsing the bytecode will result in errors.
 */
typedef struct {
	UINT16 rules_first;
	UINT16 rules_last;
	UINT16 facts_first;
	UINT16 facts_last;
	UINT16 properties_first;
	UINT16 properties_last;
} rls_header_t;

typedef struct {
	UINT16 conditions_first;
	UINT16 conditions_last;
	UINT16 statements_first;
	UINT16 statements_last;
} rule_t;

#define RLS_FACT_UNUSED		0x00
#define RLS_FACT_MODIFIED	0x01
#define RLS_FACT_PRE_MODIFIED	0x02
#define RLS_FACT_STICKY		0x04
typedef struct {
	UINT16 mac_id;
	UINT16 name;
	UINT16 properties_first;
	UINT16 properties_last;
	UINT8 state;	// A bitmask of RLS_FACT_* or RLS_FACT_UNUSED.
	// FIXME: gcc adds too much padding to the fact_t struct because of the
	// internal scatterweb_time_t struct. Size is 20 bytes, while only 16
	// are required. We have to see how the ScatterWeb compiler handles
	// this and then think of a solution.
	scatterweb_time_t timeStamp;
} fact_t;

#define RLS_VARIABLE_TYPE_BOOL		0x00
#define RLS_VARIABLE_TYPE_INTEGER	0x01
#define RLS_VARIABLE_TYPE_STRING	0x02
#define RLS_VARIABLE_TYPE_SLOT		0x03
typedef struct {
	UINT16 value;
	UINT8 type;
} variable_t;

#define RLS_PROPERTY_UNUSED	0x00	// Value of "key" field, if the property is unused in memory.
typedef struct {
	UINT16 key;
	variable_t variable;
} property_t;

#define RLS_CONDITION_TYPE_EXISTS		0x00
#define RLS_CONDITION_TYPE_EVAL			0x01
#define RLS_CONDITION_OP_EQUALS			0x00
#define RLS_CONDITION_OP_NOT_EQUALS		0x04
#define RLS_CONDITION_OP_LESS_THAN		0x08
#define RLS_CONDITION_OP_GREATER_THAN		0x0C
#define RLS_CONDITION_OP_LESS_OR_EQUAL		0x10
#define RLS_CONDITION_OP_GREATER_OR_EQUAL	0x14
#define RLS_CONDITION_MASK_TYPE			0x03	// Use to find out the type of a condition.
#define RLS_CONDITION_MASK_OP			0xFC	// Use to find out the operation in case of eval operation.
typedef struct {
	UINT16 slot_OR_left_expression;		// Depending on type.
	UINT16 UNUSED_OR_right_expression;
	UINT8 type_AND_condition_operation;	// Lower two bits for type, upper six bits for operation.
} condition_t;

#define RLS_STATEMENT_TYPE_DEFINE	0x00
#define RLS_STATEMENT_TYPE_RETRACT	0x01
#define RLS_STATEMENT_TYPE_COPY		0x02
#define RLS_STATEMENT_TYPE_SEND		0x03
#define RLS_STATEMENT_TYPE_SET		0x04
#define RLS_STATEMENT_TYPE_FLUSH	0x05
#define RLS_STATEMENT_TYPE_CALL		0x06
typedef struct {
	UINT16 name_OR_slot_OR_identifier;
	UINT16 parameters_first_OR_left_expression_OR_UNUSED;
	UINT16 parameters_last_OR_right_expression_OR_UNUSED;
	UINT8 type;
} statement_t;

#define RLS_SLOT_NAME_THIS	0x01
typedef struct {
	UINT16 factName;
	UINT16 propertyKey;
	UINT16 conditions_first;
	UINT16 conditions_last;
} slot_t;

#define RLS_EXPRESSION_TYPE_CONSTANT			0x00
#define RLS_EXPRESSION_TYPE_UNARY			0x01
#define RLS_EXPRESSION_TYPE_BINARY			0x02
#define RLS_EXPRESSION_UNARY_OP_COUNT			0x00
#define RLS_EXPRESSION_UNARY_OP_SUM			0x04
#define RLS_EXPRESSION_UNARY_OP_PRODUCT			0x08
#define RLS_EXPRESSION_UNARY_OP_MINIMUM			0x0C
#define RLS_EXPRESSION_UNARY_OP_MAXIMUM			0x10
#define RLS_EXPRESSION_UNARY_OP_SQUARE_ROOT		0x14
#define RLS_EXPRESSION_BINARY_OP_ADD			0x00
#define RLS_EXPRESSION_BINARY_OP_SUBTRACT		0x04
#define RLS_EXPRESSION_BINARY_OP_MULTIPLY		0x08
#define RLS_EXPRESSION_BINARY_OP_DIVIDE			0x0C
#define RLS_EXPRESSION_BINARY_OP_TO_THE_POWER_OF	0x10
typedef struct {
	UINT16 variable_OR_left_expression;
	UINT16 UNUSED_OR_right_expression;
	UINT8 type_AND_operation;	// Lower two bits for type, upper six bits for operation.
} expression_t;

typedef struct {
	UINT16 key;
	UINT16 expression;
} initializer_t;

/**
 * DON'T PANIC!!!
 *
 * This is just a normal function definition.
 */

void eeprom_generate(GList* rulesets, FILE* eeprom_file);

#endif // EEPROM_GEN_H
