%{

/*
 * Copyright 2005, Freie Universitaet Berlin. All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * Contributors: Georg Wittenburg
 */

#include "main.h"
#include "y.tab.h"

/**
 * This file is used by lex to generate the lexical analyzer.
 **/

%}

%option noyywrap
%x COMMENT
%%

"="				{return RC_ASSIGN;}
"("				{return RC_OPEN_ROUND_BRACKET;}
")"				{return RC_CLOSE_ROUND_BRACKET;}
"["				{return RC_OPEN_SQUARE_BRACKET;}
"]"				{return RC_CLOSE_SQUARE_BRACKET;}
"{"				{return RC_OPEN_CURLY_BRACKET;}
"}"				{return RC_CLOSE_CURLY_BRACKET;}
"->"				{return RC_RIGHT_ARROW;}
"<-"				{return RC_LEFT_ARROW;}
","				{return RC_COMMA;}
"\""				{return RC_QUOTES;}
"ruleset"			{return RC_RULESET;}
"depends"			{return RC_DEPENDS;}
"fact"				{return RC_FACT;}
"rule"				{return RC_RULE;}
"slot"				{return RC_SLOT;}
"name"				{return RC_NAME;}
"exists"			{return RC_EXISTS;}
"eval"				{return RC_EVAL;}
"define"			{return RC_DEFINE;}
"retract"			{return RC_RETRACT;}
"copy"				{return RC_COPY;}
"send"				{return RC_SEND;}
"set"				{return RC_SET;}
"flush"				{return RC_FLUSH;}
"call"				{return RC_CALL;}
"=="				{return RC_EQUALS;}
"!="				{return RC_NOT_EQUALS;}
"<"				{return RC_LESS_THAN;}
">"				{return RC_GREATER_THAN;}
"<="				{return RC_LESS_OR_EQUAL;}
">="				{return RC_GREATER_OR_EQUAL;}
"count"				{return RC_COUNT;}
"sum"				{return RC_SUM;}
"product"			{return RC_PRODUCT;}
"min"				{return RC_MINIMUM;}
"max"				{return RC_MAXIMUM;}
"sqrt"				{return RC_SQUARE_ROOT;}
"+"				{return RC_ADD;}
"-"				{return RC_SUBTRACT;}
"*"				{return RC_MULTIPLY;}
"/"				{return RC_DIVIDE;}
"^"				{return RC_TO_THE_POWER_OF;}
"true"				{yylval.int_t = 1; return RC_BOOL;}
"false"				{yylval.int_t = 0; return RC_BOOL;}
[0-9]+				{yylval.int_t = atoi(yytext); return RC_INTEGER;}
-[0-9]+				{yylval.int_t = atoi(yytext); return RC_INTEGER;}
[a-zA-Z_][a-zA-Z_\-0-9]*	{yylval.str_t = (char*) strdup(yytext); return RC_STRING;}
[ \t]+				/* consume whitespaces */
\n				{line++;}	/* next line */
\/\/.*\n			{line++;}	/* comment line */
.				{yyerror("Illegal token.");}

"/*" 				{BEGIN(COMMENT);}
<COMMENT>\n			{line++;}
<COMMENT>"*/"			{BEGIN(INITIAL);}
<COMMENT><EOF>			{yyerror("End of file while parsing multiline comment.");}
<COMMENT>.			/* consume everythnig */

%%

